@@ -4,8 +4,8 @@
 
 ;; Author: Thomas Fini Hansen <xen@xen.dk>
 ;; Created: August 11, 2019
-;; Package-Version: 20260116.2011
-;; Package-Revision: 747d53b4a7db
+;; Package-Version: 20260117.43
+;; Package-Revision: 8acd7788b4ab
 ;; Package-Requires: ((emacs "26.2"))
 ;; Keywords: tools
 ;; URL: https://github.com/xendk/reaper
@@ -610,79 +610,77 @@ Returns task id."
        (task-id (cdr (assoc (reaper--completing-read "Task: " tasks default 'reaper-task-history) tasks))))
     task-id))
 
+(defmacro reaper--with-envelope (method path payload &rest body)
+  "Run BODY with HTTP envelope.
+
+Prepare call to Harvest PATH with HTTP METHOD and PAYLOAD encoded as
+JSON."
+  `(let* ((url-request-method ,method)
+          (url-mime-language-string nil)
+          (url-mime-encoding-string nil)
+          (url-mime-accept-string "application/json")
+          (url-user-agent "Xen's Emacs client (fini@reload.dk)")
+          (url-request-data (if (or (string-equal method "POST")
+                                    (string-equal method "PATCH"))
+                                (encode-coding-string (json-encode ,payload) 'utf-8)
+                              nil))
+          (request-url (format "https://api.harvestapp.com/v2/%s" ,path))
+          (url-request-extra-headers
+           `(("Content-Type" . "application/json")
+             ("Authorization" . ,(concat "Bearer " reaper-api-key))
+             ("Harvest-Account-Id" . ,reaper-account-id))))
+     ,@body))
+
 (defun reaper-api (method path payload completion-message)
   "Make an METHOD call to PATH with PAYLOAD and COMPLETION-MESSAGE."
   (reaper--check-credentials)
-  (let* ((url-request-method method)
-         (url-mime-language-string nil)
-         (url-mime-encoding-string nil)
-         (url-mime-accept-string "application/json")
-         (url-user-agent "Xen's Emacs client (fini@reload.dk)")
-         (url-request-data (if (or (string-equal method "POST")
-                                   (string-equal method "PATCH"))
-                               (encode-coding-string (json-encode payload) 'utf-8)
-                             nil))
-         (request-url (format "https://api.harvestapp.com/v2/%s" path))
-         (url-request-extra-headers
-          `(("Content-Type" . "application/json")
-            ("Authorization" . ,(concat "Bearer " reaper-api-key))
-            ("Harvest-Account-Id" . ,reaper-account-id))))
-    (with-temp-buffer
-      (reaper-url-insert-file-contents request-url)
-      (goto-char (point-min))
-      (message "%s" completion-message)
-      ;; Ensure JSON false values is nil.
-      (defvar json-false)
-      (let ((json-false nil))
-        (json-read)))))
+  (reaper--with-envelope
+   method path payload
+   (with-temp-buffer
+     (let ((data (reaper-url-insert-file-contents request-url)))
+       (message "%s" completion-message)
+       data))))
 
-(defun reaper-url-insert-file-contents (url &optional visit beg end replace)
+(defun reaper--handle-response (buffer url)
+  "Handle response.
+
+BUFFER is the buffer to insert response into, URL is the URL fetched.
+
+Returns the decoded JSON data."
+  (when (fboundp 'url-http--insert-file-helper)
+    ;; XXX: This is HTTP/S specific and should be moved to url-http
+    ;; instead.  See bug#17549.
+    (url-http--insert-file-helper buffer url))
+  ;; TODO: check url-http-response-status
+  (url-insert-buffer-contents buffer url)
+  (goto-char (point-min))
+  ;; Ensure JSON false values is nil.
+  (defvar json-false)
+  (let ((json-false nil))
+    (json-read)))
+
+(defun reaper-url-insert-file-contents (url)
   "Quiet version of `url-insert-file-contents'.
 URL, VISIT, BEG, END and REPLACE is the same as for
 `url-insert-file-contents'."
+  ;; The extra t to `url-retrieve-synchronously' is the quiet bit.
   (let ((buffer (url-retrieve-synchronously url t)))
     (unless buffer (signal 'file-error (list url "No Data")))
-    (when (fboundp 'url-http--insert-file-helper)
-      ;; XXX: This is HTTP/S specific and should be moved to url-http
-      ;; instead.  See bug#17549.
-      (url-http--insert-file-helper buffer url visit))
-    ;; TODO: check url-http-response-status
-    (url-insert-buffer-contents buffer url visit beg end replace)))
+    (reaper--handle-response buffer url)))
 
 (defun reaper-api-async (method path payload callback)
   "Make an asynchronous HTTP call.
 
 Make a METHOD call to PATH with PAYLOAD and call CALLBACK on completion."
   (reaper--check-credentials)
-  (let* ((url-request-method method)
-         (url-mime-language-string nil)
-         (url-mime-encoding-string nil)
-         (url-mime-accept-string "application/json")
-         (url-user-agent "Xen's Emacs client (fini@reload.dk)")
-         (url-request-data (if (or (string-equal method "POST")
-                                   (string-equal method "PATCH"))
-                               (encode-coding-string (json-encode payload) 'utf-8)
-                             nil))
-         (request-url (format "https://api.harvestapp.com/v2/%s" path))
-         (url-request-extra-headers
-          `(("Content-Type" . "application/json")
-            ("Authorization" . ,(concat "Bearer " reaper-api-key))
-            ("Harvest-Account-Id" . ,reaper-account-id))))
-    (url-retrieve request-url
-                  #'(lambda (&rest _ignored)
-                      (let ((async-buffer (current-buffer)))
-                        (with-temp-buffer
-                          (when (fboundp 'url-http--insert-file-helper)
-                            ;; XXX: This is HTTP/S specific and should be moved to url-http
-                            ;; instead.  See bug#17549.
-                            (url-http--insert-file-helper async-buffer request-url))
-                          (url-insert-buffer-contents async-buffer request-url)
-                          (goto-char (point-min))
-                          ;; Ensure JSON false values is nil.
-                          (defvar json-false)
-                          (let ((json-false nil))
-                            (funcall callback (json-read))))))
-                  nil t)))
+  (reaper--with-envelope
+   method path payload
+   (url-retrieve request-url
+                 #'(lambda (&rest _ignored)
+                     (let ((async-buffer (current-buffer)))
+                       (with-temp-buffer
+                         (funcall callback (reaper--handle-response async-buffer request-url)))))
+                 nil t)))
 
 (defun reaper-alist-get (symbols alist)
   "Look up the value for the chain of SYMBOLS in ALIST."
